import json
import logging
import time
from pathlib import Path
from typing import List, Optional

import tweepy
from tqdm import tqdm

from plumes.config import settings

LOGGER = logging.getLogger("plumes")


def rate_limit_handler(cursor):
    while True:
        try:
            yield cursor.next()
        except tweepy.RateLimitError:  # pragma: no cover
            LOGGER.info(f"Rate limit exceeded; sleeping for {settings.sleep_time}s")
            time.sleep(settings.sleep_time)
        except StopIteration:
            break


def get_api():
    auth = tweepy.OAuthHandler(settings.CONSUMER_KEY, settings.CONSUMER_SECRET)
    auth.set_access_token(settings.ACCESS_TOKEN, settings.ACCESS_TOKEN_SECRET)
    api = tweepy.API(auth)

    return api


def set_output(fname: str, path: Optional[str]):
    if path:
        path = Path(path)

        if not path.exists():  # pragma: no cover
            path.mkdir(parents=True, exist_ok=True)

        if path.is_dir():
            path / fname

    else:
        path = Path.cwd() / fname

    LOGGER.info(f"Output path: {path.resolve()}")

    return path


def get_tweepy_objects(
    func, screen_name: str, output: Path, total: int, count: int = 200
):
    # get users
    objs = []
    with tqdm(total=total) as pbar:
        for o in rate_limit_handler(
            tweepy.Cursor(func, screen_name=screen_name, count=count).items(total)
        ):
            objs.append(o)
            pbar.update(1)

    # dump output
    tweepy_to_json(models=objs, path=output)


def tweepy_to_json(models: List, path: Path):
    models = [m._json for m in models]
    with open(path, "w") as f:
        json.dump(models, f, indent=4)


def get_user(screen_name: Optional[str] = None):
    if screen_name:
        user = get_api().get_user(screen_name)
    else:
        user = get_api().me()
    return user


def calculate_like_retweet_ratio(likes: int, retweets: int) -> float:
    if likes == 0:
        ratio = 0
    elif retweets == 0:
        ratio = float("inf")
    else:
        ratio = likes / retweets

    return ratio


def calculate_tff_ratio(followers: int, friends: int) -> float:
    if followers == 0:
        ratio = 0
    elif friends == 0:
        ratio = float("inf")
    else:
        ratio = followers / friends

    return ratio



from pathlib import Path

from dynaconf import Dynaconf

user_config_path = Path.home() / ".plumes.toml"
package_config_path = Path(__file__).parent / "settings.toml"
settings = Dynaconf(
    envvar_prefix="PLUMES", settings_files=[user_config_path, package_config_path]
)



import datetime
import email.utils as eu
import json
import logging
import logging.config
import textwrap
from pathlib import Path
from typing import Optional

import fire
import toml
import tweepy

import plumes.utilities as pu
from plumes.config import settings, user_config_path

logging.config.dictConfig(settings.logging)
LOGGER = logging.getLogger("plumes")


def init(force: bool = False, path: Optional[str] = None):
    """Initialize your plumes configuration

    Args:
        force (bool, optional): Force overwrite of existing config file. Defaults to False.
    """

    if not path:
        path = user_config_path
    else:
        path = Path(path)

    # path exists and force option wasn't used
    if not force and path.exists():  # pragma: no cover
        LOGGER.info(
            f"{path.resolve()} exists. Please use the `--force` flag if you want to overwrite."
        )
    else:
        LOGGER.info(f"Creating template config file: {path.resolve()}")
        with open(path, "w") as f:
            toml.dump(settings.default.config, f)
        LOGGER.info(
            (
                "Blank configuration created. "
                f"Please visit {settings.project_homepage} for more info. "
                f"Please visit {settings.twitter_dev_page} for your API tokens."
            )
        )


def check_config():  # pragma: no cover
    """Check and validate the current configuation.
    """
    # check env variables
    LOGGER.info("Checking config settings")
    is_valid_config = True
    for k in settings.default.config:
        if k in settings.as_dict():
            LOGGER.info(f"{k.upper()} found in settings")
        else:
            LOGGER.warning(f"{k.upper()} not found in settings!")
            is_valid_config = False

    # special message if a bad config is detected
    bad_config_message = (
        "Invalid configuration. "
        f"Please visit {settings.project_homepage} for more info. "
        f"Please visit {settings.twitter_dev_page} for your API tokens."
    )
    if not is_valid_config:
        LOGGER.warning(bad_config_message)
        return

    # check authentication
    LOGGER.info("Checking Twitter authentication")
    try:
        api = pu.get_api()
        me = api.me()
        LOGGER.info(f"Successfully authenticated as {me.screen_name}")
    except tweepy.error.TweepError:
        LOGGER.warning("Invalid authentication values!")
        LOGGER.warning(bad_config_message)


def view_config():  # pragma: no cover
    """Print the current configuration
    """
    print(
        json.dumps(
            settings.as_dict(),
            indent=4,
            sort_keys=True,
            default=lambda o: "<not serializable>",
        )
    )


def friends(
    screen_name: Optional[str] = None,
    limit: Optional[int] = None,
    output: Optional[str] = None,
):
    """Get JSON array of friends

    Args:
        screen_name (Optional[str], optional): Target user's screen name (i.e., Twitter handle). If none is given, authenticated user is used. Defaults to None.
        limit (Optional[int], optional): Max number of users to fetch. Defaults to None.
        output (Optional[str], optional): Output path for JSON file. Defaults to None.
    """
    # get api and user object
    api = pu.get_api()
    source_user = pu.get_user(screen_name=screen_name)

    # check limit for progress bar
    if not limit:  # pragma: no cover
        limit = source_user.friends_count

    # ensure output location
    fname = f"{source_user.screen_name}-friends.json"
    path = pu.set_output(fname=fname, path=output)

    # get users
    LOGGER.info(f"Fetching {limit} friends")
    pu.get_tweepy_objects(
        func=api.friends, screen_name=screen_name, output=path, total=limit
    )


def followers(
    screen_name: Optional[str] = None,
    limit: Optional[int] = None,
    output: Optional[str] = None,
):
    """Get JSON array of followers

    Args:
        screen_name (Optional[str], optional): Target user's screen name (i.e., Twitter handle). If none is given, authenticated user is used. Defaults to None.
        limit (Optional[int], optional): Max number of users to fetch. Defaults to None.
        output (Optional[str], optional): Output path for JSON file. Defaults to None.
    """
    # get api and user object
    api = pu.get_api()
    source_user = pu.get_user(screen_name=screen_name)

    # check limit for progress bar
    if not limit:  # pragma: no cover
        limit = source_user.followers_count

    # ensure output location
    fname = f"{source_user.screen_name}-followers.json"
    path = pu.set_output(fname=fname, path=output)

    # get users
    LOGGER.info(f"Fetching {limit} followers")
    pu.get_tweepy_objects(
        func=api.followers, screen_name=screen_name, output=path, total=limit
    )


def favorites(
    screen_name: Optional[str] = None,
    limit: Optional[int] = None,
    output: Optional[str] = None,
):
    """Get JSON array of favourited tweets.

    Args:
        screen_name (Optional[str], optional): Target user's screen name (i.e., Twitter handle). If none is given, authenticated user is used. Defaults to None.
        limit (Optional[int], optional): Max number of users to fetch. Defaults to None.
        output (Optional[str], optional): Output path for JSON file. Defaults to None.
    """
    # get api and user object
    api = pu.get_api()
    source_user = pu.get_user(screen_name=screen_name)

    # check limit for progress bar
    if not limit:  # pragma: no cover
        limit = source_user.statuses_count

    # ensure output location
    fname = f"{source_user.screen_name}-favorites.json"
    path = pu.set_output(fname=fname, path=output)

    # get tweets
    LOGGER.info(f"Fetching {limit} favourited tweets")
    pu.get_tweepy_objects(
        func=api.favorites, screen_name=screen_name, output=path, total=limit
    )


def tweets(
    screen_name: Optional[str] = None,
    limit: Optional[int] = None,
    output: Optional[str] = None,
):
    """Get JSON array of tweets

    Args:
        screen_name (Optional[str], optional): Target user's screen name (i.e., Twitter handle). If none is given, authenticated user is used. Defaults to None.
        limit (Optional[int], optional): Max number of users to fetch. Defaults to None.
        output (Optional[str], optional): Output path for JSON file. Defaults to None.
    """
    # get api and user object
    api = pu.get_api()
    source_user = pu.get_user(screen_name=screen_name)

    # check limit for progress bar
    if not limit:  # pragma: no cover
        limit = source_user.statuses_count

    # ensure output location
    fname = f"{source_user.screen_name}-tweets.json"
    path = pu.set_output(fname=fname, path=output)

    # get tweets
    LOGGER.info(f"Fetching {limit} tweets")
    pu.get_tweepy_objects(
        func=api.user_timeline, screen_name=screen_name, output=path, total=limit
    )


def audit_users(  # noqa C901
    path: str,
    min_followers: Optional[int] = None,
    max_followers: Optional[int] = None,
    min_friends: Optional[int] = None,
    max_friends: Optional[int] = None,
    days: Optional[int] = None,
    min_tweets: Optional[int] = None,
    max_tweets: Optional[int] = None,
    min_favourites: Optional[int] = None,
    max_favourites: Optional[int] = None,
    min_ratio: Optional[float] = None,
    max_ratio: Optional[float] = None,
    prune: bool = False,
    befriend: bool = False,
    bool_or: bool = False,
):
    """Audit and review users given criteria

    Args:
        path (str): Path to JSON file of users (e.g., output of friends())
        min_followers (Optional[int], optional): Min number of followers. Defaults to None.
        max_followers (Optional[int], optional): Max number of followers. Defaults to None.
        min_friends (Optional[int], optional): Min number of friends. Defaults to None.
        max_friends (Optional[int], optional): Max number of friends. Defaults to None.
        days (Optional[int], optional): Days since last tweet. Defaults to None.
        min_tweets (Optional[int], optional): Min number of tweets. Defaults to None.
        max_tweets (Optional[int], optional): Max number of tweets. Defaults to None.
        min_favourites (Optional[int], optional): Min number of favourites. Defaults to None.
        max_favourites (Optional[int], optional): Max number of favourites. Defaults to None.
        min_ratio (Optional[float], optional): Min Twitter follower-friend (TFF) ratio. Defaults to None.
        max_ratio (Optional[float], optional): Max Twitter follower-friend (TFF) ratio. Defaults to None.
        prune (bool, optional): Unfollow identified users. Defaults to False.
        befriend (bool, optional): Follow identified users. Defaults to False.
        bool_or (bool, optional): Switch to boolean OR for conditions. Defaults to False.
    """

    # load users data
    path = Path(path)
    with open(path) as f:
        users = json.load(f)
    LOGGER.info(f"Loaded {len(users)} users")

    # init list of users to prune
    identified_users = set()

    # iterate and identify prunable users
    for u in sorted(users, key=lambda x: x["screen_name"].lower()):
        failed_clauses = []

        if min_followers is not None:
            failed_clauses.append(u["followers_count"] < min_followers)

        if max_followers is not None:
            failed_clauses.append(u["followers_count"] > max_followers)

        if min_friends is not None:
            failed_clauses.append(u["friends_count"] < min_friends)

        if max_friends is not None:
            failed_clauses.append(u["friends_count"] > max_friends)

        if days is not None:
            today = datetime.date.today()
            limit = today - datetime.timedelta(days=days)
            try:
                last_tweet = eu.parsedate_to_datetime(u["status"]["created_at"])
                failed_clauses.append(last_tweet.date() < limit)
            except KeyError:
                # no tweets
                failed_clauses.append(True)

        if min_tweets is not None:
            failed_clauses.append(u["statuses_count"] < min_tweets)

        if max_tweets is not None:
            failed_clauses.append(u["statuses_count"] > max_tweets)

        if min_favourites is not None:
            failed_clauses.append(u["favourites_count"] < min_favourites)

        if max_favourites is not None:
            failed_clauses.append(u["favourites_count"] > max_favourites)

        if min_ratio is not None:
            failed_clauses.append(
                pu.calculate_tff_ratio(
                    followers=u["followers_count"], friends=u["friends_count"]
                )
                < min_ratio
            )

        if max_ratio is not None:
            failed_clauses.append(
                pu.calculate_tff_ratio(
                    followers=u["followers_count"], friends=u["friends_count"]
                )
                > max_ratio
            )

        if len(failed_clauses) > 0:
            if (bool_or and any(failed_clauses)) or all(failed_clauses):
                LOGGER.info(f"Identified {u['screen_name']}")
                identified_users.add(u["screen_name"])

    LOGGER.info(f"Identified {len(identified_users)} users")
    if prune:  # pragma: no cover
        for u in identified_users:
            LOGGER.info(f"Unfollowing {u}")

            try:
                pu.get_api().destroy_friendship(u)
            except tweepy.error.TweepError as e:
                LOGGER.error(e)
    if befriend:  # pragma: no cover
        for u in identified_users:
            LOGGER.info(f"Following {u}")
            try:
                pu.get_api().create_friendship(u)
            except tweepy.error.TweepError as e:
                LOGGER.error(e)


def audit_tweets(  # noqa C901
    path: str,
    days: Optional[int] = None,
    min_likes: Optional[int] = None,
    max_likes: Optional[int] = None,
    min_retweets: Optional[int] = None,
    max_retweets: Optional[int] = None,
    min_ratio: Optional[float] = None,
    max_ratio: Optional[float] = None,
    self_favorited: Optional[bool] = None,
    prune: bool = False,
    favorite: bool = False,
    bool_or: bool = False,
):
    """Audit and review tweets given criteria

    Args:
        days (Optional[int], optional): Days since tweeted. Defaults to None.
        min_likes (Optional[int], optional): Min number of favourites. Defaults to None.
        max_likes (Optional[int], optional): Max number of favourites. Defaults to None.
        min_retweets (Optional[int], optional): Min number of retweets. Defaults to None.
        max_retweets (Optional[int], optional): Max number of retweets. Defaults to None.
        min_ratio (Optional[float], optional): Min Twitter like-retweet ratio. Defaults to None.
        max_ratio (Optional[float], optional): Max Twitter like-retweet ratio. Defaults to None.
        self_favorited (Optional[bool], optional): Check if tweet is self-liked. Defaults to None.
        prune (bool, optional): Prune and destroy identified tweets. Defaults to False.
        favorite (bool, optional): Like identified tweets. Defaults to False.
        bool_or (bool, optional): Switch to boolean OR for conditions. Defaults to False.
    """
    # load data
    path = Path(path)
    with open(path) as f:
        tweets = json.load(f)
    LOGGER.info(f"Loaded {len(tweets)} tweets")

    # init set of users to prune
    identified_tweets = set()

    # iterate and identify prunable users
    for t in tweets:
        text = textwrap.shorten(t["text"], width=settings.textwrap_width)
        failed_clauses = []

        if days is not None:
            today = datetime.date.today()
            limit = today - datetime.timedelta(days=days)
            tweet_dt = eu.parsedate_to_datetime(t["created_at"])
            failed_clauses.append(tweet_dt.date() < limit)

        if min_likes is not None:
            failed_clauses.append(t["favorite_count"] < min_likes)

        if max_likes is not None:
            failed_clauses.append(t["favorite_count"] > max_likes)

        if min_retweets is not None:
            failed_clauses.append(t["retweet_count"] < min_retweets)

        if max_retweets is not None:
            failed_clauses.append(t["retweet_count"] > max_retweets)

        if min_ratio is not None:
            failed_clauses.append(
                pu.calculate_like_retweet_ratio(
                    likes=t["favorite_count"], retweets=t["retweet_count"]
                )
                < min_ratio
            )

        if max_ratio is not None:
            failed_clauses.append(
                pu.calculate_like_retweet_ratio(
                    likes=t["favorite_count"], retweets=t["retweet_count"]
                )
                > max_ratio
            )

        if self_favorited is not None:
            failed_clauses.append(t["favorited"] == self_favorited)

        if len(failed_clauses) > 0:
            if (bool_or and any(failed_clauses)) or all(failed_clauses):
                LOGGER.info(f'Identified "{text}"')
                identified_tweets.add(t["id_str"])

    LOGGER.info(f"Identified {len(identified_tweets)} tweets")
    if prune:  # pragma: no cover
        for t in identified_tweets:
            LOGGER.info(f"Deleting {t}")

            try:
                pu.get_api().destroy_status(t)
            except tweepy.error.TweepError as e:
                LOGGER.error(e)
    if favorite:  # pragma: no cover
        for t in identified_tweets:
            LOGGER.info(f"Favoriting {t}")
            pu.get_api().create_favorite(t)


def view_user(user: str):
    """View a user's raw JSON

    Args:
        user (str): User's screen name
    """
    user = pu.get_api().get_user(user)
    print(
        json.dumps(
            user._json,
            indent=4,
            sort_keys=True,
            default=lambda o: "<not serializable>",
        )
    )


def main():  # pragma: no cover
    fire.Fire()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['plumes']

package_data = \
{'': ['*']}

install_requires = \
['dynaconf>=3.1.0,<4.0.0',
 'fire>=0.3.1,<0.4.0',
 'python-box>=5.1.0,<6.0.0',
 'toml>=0.10.1,<0.11.0',
 'tqdm>=4.48.2,<5.0.0',
 'tweepy>=3.9.0,<4.0.0']

entry_points = \
{'console_scripts': ['plumes = plumes.cli:main']}

setup_kwargs = {
    'name': 'plumes',
    'version': '2.3.2',
    'description': 'Simple Twitter CLI for day-to-day social media hygiene',
    'long_description': '<br />\n<p align="center">\n  <a href="https://github.com/nnadeau/plumes">\n    <img src="https://raw.githubusercontent.com/nnadeau/plumes/master/media/feather.png" alt="Logo" width="80" height="80">\n  </a>\n\n  <h3 align="center">plumes</h3>\n\n  <p align="center">\n    Simple Twitter CLI for day-to-day social media hygiene\n    <br />\n    ·\n    <a href="https://github.com/nnadeau/plumes/issues">Report Bug</a>\n    ·\n    <a href="https://github.com/nnadeau/plumes/issues">Request Feature</a>\n  </p>\n</p>\n\n[![GitHub issues](https://img.shields.io/github/issues/nnadeau/plumes)](https://github.com/nnadeau/plumes/issues)\n[![GitHub forks](https://img.shields.io/github/forks/nnadeau/plumes)](https://github.com/nnadeau/plumes/network)\n[![GitHub stars](https://img.shields.io/github/stars/nnadeau/plumes)](https://github.com/nnadeau/plumes/stargazers)\n[![GitHub license](https://img.shields.io/github/license/nnadeau/plumes)](https://github.com/nnadeau/plumes/blob/master/LICENSE)\n\n[![PyPI Version](https://img.shields.io/pypi/v/plumes.svg)](https://pypi.python.org/pypi/plumes)\n[![PyPI License](https://img.shields.io/pypi/l/plumes.svg)](https://pypi.python.org/pypi/plumes)\n[![PyPI Wheel](https://img.shields.io/pypi/wheel/plumes.svg)](https://pypi.python.org/pypi/plumes)\n[![PyPI Format](https://img.shields.io/pypi/format/plumes.svg)](https://pypi.python.org/pypi/plumes)\n[![PyPI Pythons](https://img.shields.io/pypi/pyversions/plumes.svg)](https://pypi.python.org/pypi/plumes)\n[![PyPI Implementation](https://img.shields.io/pypi/implementation/plumes.svg)](https://pypi.python.org/pypi/plumes)\n\n[![semantic-release](https://img.shields.io/badge/%20%20%F0%9F%93%A6%F0%9F%9A%80-semantic--release-e10079.svg)](https://github.com/semantic-release/semantic-release)\n[![Test](https://github.com/nnadeau/plumes/workflows/Test/badge.svg)](https://github.com/nnadeau/plumes/actions)\n[![Release](https://github.com/nnadeau/plumes/workflows/Release/badge.svg)](https://github.com/nnadeau/plumes/actions)\n[![Publish](https://github.com/nnadeau/plumes/workflows/Publish/badge.svg)](https://github.com/nnadeau/plumes/actions)\n\n[![Twitter](https://img.shields.io/twitter/url?style=social&url=https%3A%2F%2Fgithub.com%2Fnnadeau%2Fplumes)](https://twitter.com/intent/tweet?text=Wow:&url=https%3A%2F%2Fgithub.com%2Fnnadeau%2Fplumes)\n\n## Contents\n\n- [Contents](#contents)\n- [Overview](#overview)\n- [Inspiration](#inspiration)\n- [Installation](#installation)\n- [Usage](#usage)\n  - [Getting Started And Creating Your Authentication Config](#getting-started-and-creating-your-authentication-config)\n  - [Export Friends](#export-friends)\n  - [Export Followers](#export-followers)\n  - [Export Tweets](#export-tweets)\n  - [Audit Users](#audit-users)\n  - [Prune Your Tweets](#prune-your-tweets)\n- [Setting Up Authentication](#setting-up-authentication)\n  - [Get Your Twitter API Tokens](#get-your-twitter-api-tokens)\n  - [Configuring `plumes`](#configuring-plumes)\n- [Contributing](#contributing)\n- [Testing](#testing)\n\n## Overview\n\n`Plumes` is an open-source Python CLI app for day-to-day social media hygiene.\nIt was designed to provide a simple, clear, and concise interface to quickly explore and clean a personal Twitter account.\n\n## Inspiration\n\n[![panzer tweet](https://raw.githubusercontent.com/nnadeau/plumes/master/media/tweet-panzer.png)](https://twitter.com/panzer/status/943935357673861120)\n\n[![chrisalbon tweet](https://raw.githubusercontent.com/nnadeau/plumes/master/media/tweet-chrisalbon.png)](https://twitter.com/chrisalbon/status/1295408107078615041)\n\n\n## Installation\n\n```bash\n# python >=3.6.1 is required\npip install plumes\n```\n\n## Usage\n\n### Getting Started And Creating Your Authentication Config\n\n```bash\n# create your config file\nplumes init\n\n# validate your config file\nplumes check_config\n\n# print your config file (watch out for sensitive tokens!)\nplumes view_config\n```\n\n### Export Friends\n\n[Extract friends](examples/SteveMartinToGo-friends.json) ordered in which they were added:\n\n```bash\nplumes friends <flags>\n\n# e.g., get the friends of Steve Martin (see data in examples dir)\nplumes friends SteveMartinToGo --limit 100\n```\n\n**Arguments**:\n\n- `screen_name` _Optional[str], optional_ - Target user\'s screen name (i.e., Twitter handle). If none is given, authenticated user is used. Defaults to None.\n- `limit` _Optional[int], optional_ - Max number of users to fetch. Defaults to None.\n- `output` _Optional[str], optional_ - Output path for JSON file. Defaults to None.\n\n![Plumes friends gif](https://raw.githubusercontent.com/nnadeau/plumes/master/media/terminal-friends.gif)\n\n### Export Followers\n\n[Extract followers](examples/alyankovic-followers.json) ordered in which they were added:\n\n```bash\nplumes followers <flags>\n\n# e.g., get the followers of Al Yankovic (see data in examples dir)\nplumes followers alyankovic --limit 100\n```\n\n**Arguments**:\n\n- `screen_name` _Optional[str], optional_ - Target user\'s screen name (i.e., Twitter handle). If none is given, authenticated user is used. Defaults to None.\n- `limit` _Optional[int], optional_ - Max number of users to fetch. Defaults to None.\n- `output` _Optional[str], optional_ - Output path for JSON file. Defaults to None.\n\n### Export Tweets\n\n[Extract (and archive) tweets](examples/ConanOBrien-tweets.json) ordered in by most recent:\n\n```bash\nplumes tweets <flags>\n\n# e.g., get the tweets of Conan O\'Brien (see data in examples dir)\nplumes tweets ConanOBrien --limit 100\n```\n\n**Arguments**:\n\n- `screen_name` _Optional[str], optional_ - Target user\'s screen name (i.e., Twitter handle). If none is given, authenticated user is used. Defaults to None.\n- `limit` _Optional[int], optional_ - Max number of users to fetch. Defaults to None.\n- `output` _Optional[str], optional_ - Output path for JSON file. Defaults to None.\n\n![Plumes tweet gif](https://raw.githubusercontent.com/nnadeau/plumes/master/media/terminal-tweets.gif)\n\n### Audit Users\n\nAudit and review users given criteria.\nUse this to mass follow/unfollow many users.\n\n```bash\nplumes audit_users PATH <flags>\n\n# e.g., follow 100 of Al Yankovic\'s followers\nplumes followers alyankovic --limit 100\nplumes audit_users alyankovic-followers.json --befriend\n\n# e.g., prune (i.e., unfollow) current friends who have less than 100 followers AND haven\'t tweeted in the last 30 days\nplumes friends --output "friends.json"\nplumes audit_users "friends.json" --prune --min_followers 100 --days 30\n```\n\n**Arguments**:\n\n- `path` _str_ - Path to JSON file of users (e.g., output of friends())\n- `min_followers` _Optional[int], optional_ - Min number of followers. Defaults to None.\n- `max_followers` _Optional[int], optional_ - Max number of followers. Defaults to None.\n- `min_friends` _Optional[int], optional_ - Min number of friends. Defaults to None.\n- `max_friends` _Optional[int], optional_ - Max number of friends. Defaults to None.\n- `days` _Optional[int], optional_ - Days since last tweet. Defaults to None.\n- `min_tweets` _Optional[int], optional_ - Min number of tweets. Defaults to None.\n- `max_tweets` _Optional[int], optional_ - Max number of tweets. Defaults to None.\n- `min_favourites` _Optional[int], optional_ - Min number of favourites. Defaults to None.\n- `max_favourites` _Optional[int], optional_ - Max number of favourites. Defaults to None.\n- `min_ratio` _Optional[float], optional_ - Min Twitter follower-friend (TFF) ratio. Defaults to None.\n- `max_ratio` _Optional[float], optional_ - Max Twitter follower-friend (TFF) ratio. Defaults to None.\n- `prune` _bool, optional_ - Unfollow identified users. Defaults to False.\n- `befriend` _bool, optional_ - Follow identified users. Defaults to False.\n- `bool_or` _bool, optional_ - Switch to boolean OR for conditions. Defaults to False.\n\n### Prune Your Tweets\n\nAudit and review tweets given criteria.\nUse this to mass favourite or delete tweets.\n\n```bash\nplumes audit_tweets PATH <flags>\n\n# e.g., delete your tweets that are older than 60 days AND that you didn\'t self-favourite\nplumes tweets --output "tweets.json"\nplumes audit_tweets "tweets.json" --prune --days 60 --self_favorited False\n\n# e.g., export 100 of Conan O\'Brien\'s tweets and favourite those that have a maximum of 10 likes and a minimum of 50 retweets\nplumes tweets ConanOBrien --limit 100\nplumes audit_tweets ConanOBrien-tweets.json --favorite --max_likes 10 --min_retweets 50\n```\n\n**Arguments**:\n\n- `days` _Optional[int], optional_ - Days since tweeted. Defaults to None.\n- `min_likes` _Optional[int], optional_ - Min number of favourites. Defaults to None.\n- `max_likes` _Optional[int], optional_ - Max number of favourites. Defaults to None.\n- `min_retweets` _Optional[int], optional_ - Min number of retweets. Defaults to None.\n- `max_retweets` _Optional[int], optional_ - Max number of retweets. Defaults to None.\n- `min_ratio` _Optional[float], optional_ - Min Twitter like-retweet ratio. Defaults to None.\n- `max_ratio` _Optional[float], optional_ - Max Twitter like-retweet ratio. Defaults to None.\n- `self_favorited` _Optional[bool], optional_ - Check if tweet is self-liked. Defaults to None.\n- `prune` _bool, optional_ - Prune and destroy identified tweets. Defaults to False.\n- `favorite` _bool, optional_ - Like identified tweets. Defaults to False.\n- `bool_or` _bool, optional_ - Switch to boolean OR for conditions. Defaults to False.\n\n## Setting Up Authentication\n\n### Get Your Twitter API Tokens\n\n- Navigate to the [Twitter Dev Portal](https://developer.twitter.com/en/apps)\n- Click `Create an app`\n\n  ![Twitter app link](https://raw.githubusercontent.com/nnadeau/plumes/master/media/2020-08-20-09-11-05.png)\n\n- Fill out the forms\n- Navigate to `Keys and tokens`\n\n  ![Tokens link](https://raw.githubusercontent.com/nnadeau/plumes/master/media/2020-08-20-09-12-34.png)\n\n- Copy the values for `API key`, `API secret key`, `Access token`, and `Access token secret`\n\n  ![Keys and tokens](https://raw.githubusercontent.com/nnadeau/plumes/master/media/2020-08-20-09-13-24.png)\n\n### Configuring `plumes`\n\nThe API tokens can either be set as environment variables (using the `PLUMES_` prefix; e.g., `export PLUMES_CONSUMER_KEY=xxxxx`) or configuration variables in `~/.plumes.toml`:\n\n- CONSUMER_KEY = `<API key>`\n- CONSUMER_SECRET = `<API secret key>`\n- ACCESS_TOKEN = `<Access token>`\n- ACCESS_TOKEN_SECRET = `<Access token secret>`\n\n## Contributing\n\nPlease see [`CONTRIBUTING.md`](.github/CONTRIBUTING.md) and the [Code of Conduct](CODE_OF_CONDUCT.md) for how to contribute to the project\n\n## Testing\n\n- Please review the [`Makefile`](Makefile) for an overview of all available tests\n- The most important tests and `make` commands are highlighted below:\n\n```bash\n# auto-format code\nmake format\n\n# perform all static tests\nmake check\n```\n\n---\n\n<div>Icons made by <a href="https://smashicons.com/" title="Smashicons">Smashicons</a> from <a href="https://www.flaticon.com/" title="Flaticon">www.flaticon.com</a></div>\n',
    'author': 'Nicholas Nadeau',
    'author_email': 'nicholas.nadeau@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/nnadeau/plumes',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
